#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// </summary>
    [Description("Gaussian Filter Ported to NT")]
    [Gui.Design.DisplayName("RGaussianFilter")]
    public class RGaussianFilter : Indicator
    {
        #region Variables
        // Wizard generated variables
    	private int period     	= 90;
		private double w		= 0;
    	private double y1 		= 0;
    	private double aa		= 0;
    	private double y		= 0;
    	private double y2		= 0;
    	private double sqrtOf2	= Math.Sqrt(2.0);
    	private double y3		= 0;
    	private double y4		= 0;
    	private double a2		= 0;
    	private double a3		= 0;
    	private double a4		= 0;
    	private double Pi		= 22/7;
    	private double a_12		= 0;
    	private int poles		= 2;
    	private double b		= 0;
    	private double a_13		= 0;
    	private double a_14		= 0;
    	private double a_1		= 0;
		private double Price;	
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.Red, PlotStyle.Line, "Gauss"));
            
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= true;
        }

		/// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			Price = Input[0];
			if(CurrentBar == 0)
			{
				w = 2 * Pi / Period;
//				w = 180 * w / Pi;
    	        b = (1 - Math.Cos(w)) / (Math.Pow(sqrtOf2, 2.0 / poles) - 1);
        	    aa = -b + Math.Sqrt(b * b + 2 * b);
            	a_1 = 1.0 - aa;
	            a_12 = a_1 * a_1;
	            a_13 = a_1 * a_1 * a_1;
	            a_14 = a_12 * a_12;
				a2 = aa * aa;
				a3 = aa * aa * aa;
				a4 = a2 * a2;
				y1 = Price;
				y2 = y1;
				y3 = y2;
				y4 = y3;
			}

			if (CurrentBar < Period) 
			{	
				return;
			}

			switch (poles)
			{
				case 1:
					y = aa * Price + a_1 * y1;  break;
            	case 2:
					y = a2 * Price + 2 * a_1 * y1 - a_12 * y2;  break;
				case 3:
					y = a3 * Price + 3 * a_1 * y1 - 3 * a_12 * y2 + a_13 * y3;  break;
				case 4:
					y = a4 * Price + 4 * a_1 * y1 - 6 * a_12 * y2 + 4 * a_13 * y3 - a_14 * y4; break;
            }
			if (FirstTickOfBar)
			{
				y4 = y3;
         	   	y3 = y2;
         	   	y2 = y1;
         	   	y1 = y;
			}
            Gauss.Set(y);
        }
			

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Gauss
        {
            get { return Values[0]; }
        }

        [Description("")]
    	[Category("Parameters")]
	    public int Poles
    	{
        get {return poles; }
        set { poles = Math.Min(Math.Max(1, value),4); }
        }
       
    
    [Description("")]
    [Category("Parameters")]
    public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        
    #endregion

	}
}
	
      


#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RGaussianFilter[] cacheRGaussianFilter = null;

        private static RGaussianFilter checkRGaussianFilter = new RGaussianFilter();

        /// <summary>
        /// Gaussian Filter Ported to NT
        /// </summary>
        /// <returns></returns>
        public RGaussianFilter RGaussianFilter(int period, int poles)
        {
            return RGaussianFilter(Input, period, poles);
        }

        /// <summary>
        /// Gaussian Filter Ported to NT
        /// </summary>
        /// <returns></returns>
        public RGaussianFilter RGaussianFilter(Data.IDataSeries input, int period, int poles)
        {
            checkRGaussianFilter.Period = period;
            period = checkRGaussianFilter.Period;
            checkRGaussianFilter.Poles = poles;
            poles = checkRGaussianFilter.Poles;

            if (cacheRGaussianFilter != null)
                for (int idx = 0; idx < cacheRGaussianFilter.Length; idx++)
                    if (cacheRGaussianFilter[idx].Period == period && cacheRGaussianFilter[idx].Poles == poles && cacheRGaussianFilter[idx].EqualsInput(input))
                        return cacheRGaussianFilter[idx];

            RGaussianFilter indicator = new RGaussianFilter();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.Poles = poles;
            indicator.SetUp();

            RGaussianFilter[] tmp = new RGaussianFilter[cacheRGaussianFilter == null ? 1 : cacheRGaussianFilter.Length + 1];
            if (cacheRGaussianFilter != null)
                cacheRGaussianFilter.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRGaussianFilter = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Gaussian Filter Ported to NT
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RGaussianFilter RGaussianFilter(int period, int poles)
        {
            return _indicator.RGaussianFilter(Input, period, poles);
        }

        /// <summary>
        /// Gaussian Filter Ported to NT
        /// </summary>
        /// <returns></returns>
        public Indicator.RGaussianFilter RGaussianFilter(Data.IDataSeries input, int period, int poles)
        {
            return _indicator.RGaussianFilter(input, period, poles);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Gaussian Filter Ported to NT
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RGaussianFilter RGaussianFilter(int period, int poles)
        {
            return _indicator.RGaussianFilter(Input, period, poles);
        }

        /// <summary>
        /// Gaussian Filter Ported to NT
        /// </summary>
        /// <returns></returns>
        public Indicator.RGaussianFilter RGaussianFilter(Data.IDataSeries input, int period, int poles)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RGaussianFilter(input, period, poles);
        }

    }
}
#endregion
