#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

public enum SqueezeStyle
{
	BBSqueeze,
	PBFSqueeze,
	CounterTrend
}
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("PBF and BB Squeeze Replica")]
    public class RSqueeze : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int length = 20; // Default setting for Lenght
            private double nBB = 2; // Default setting for NBB
            private double nK = 1.5; // Default setting for NK
            private int mLength = 20; // Default setting for MLenght
			private SqueezeStyle sStyle = SqueezeStyle.BBSqueeze;
			private DataSeries MyValue, MyValue2; 
			private	double fv1, sv1, fv2, sv2, fv3, sv3, fv4, sv4, d1, d2, d3, d4, avd;
			private int upct, downct;
			private Color alertcolor = Color.Red;
			private Color normalcolor = Color.Blue;
			private Color pmup = Color.Lime;		
			private Color pmdown = Color.DarkGreen;
			private Color nmdown = Color.Red;
			private Color nmup = Color.Maroon;
			private Color pbfbuy = Color.DarkGreen;
			private Color pbfsell = Color.DarkRed;
			private int zerolinewidth = 2;
			private int histogramwidth = 5;
			private int cciperiod = 13;
			private int gaussperiod = 21;
			private int gausspoles = 3;
			private int gausspolescounter = 4;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(alertcolor, PlotStyle.Dot, "Squeeze"));
            Add(new Plot(normalcolor, PlotStyle.Dot, "SqueezeOn"));
            Add(new Plot(pmup, PlotStyle.Bar, "PMomentumUp"));
            Add(new Plot(pmdown, PlotStyle.Bar, "PMomentumDown"));
            Add(new Plot(nmup, PlotStyle.Bar, "NMomentumUp"));
            Add(new Plot(nmdown, PlotStyle.Bar, "NMomentumDown"));
			MyValue = new DataSeries(this);
			MyValue2 = new DataSeries(this);

            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			PlotsConfigurable   = false;
			
			Plots[0].Pen.Width = zerolinewidth;
			Plots[1].Pen.Width = zerolinewidth;
 			Plots[2].Pen.Width = histogramwidth;
			Plots[3].Pen.Width = histogramwidth;
			Plots[4].Pen.Width = histogramwidth;
			Plots[5].Pen.Width = histogramwidth;
       }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			Plots[2].Pen = new Pen(pmup, histogramwidth);
			Plots[3].Pen = new Pen(pmdown, histogramwidth);
			Plots[4].Pen = new Pen(nmup, histogramwidth);
			Plots[5].Pen = new Pen(nmdown, histogramwidth);
			switch (sStyle)
			{
				case SqueezeStyle.BBSqueeze:
					
					double Avtrrg, SD, BBSInd;
					Plots[0].Pen = new Pen(alertcolor, zerolinewidth);
					Plots[1].Pen = new Pen(normalcolor, zerolinewidth);
					if (CurrentBar >0)
					{
						Avtrrg = ATR(length)[0];
						SD = StdDev(Close, length)[0];
						if ((nK*Avtrrg)!= 0) BBSInd = (nBB*SD)/(nK*Avtrrg); else BBSInd = 1;
						if (BBSInd <= 1) Squeeze.Set(0); else	SqueezeOn.Set(0);
						MyValue2.Set(Input[0]-(((DonchianChannel(Input,mLength).Mean.Get(base.CurrentBar))+(EMA(Input,mLength)[0]))/2));
						MyValue.Set(LinReg(MyValue2,mLength)[0]);
						if (MyValue[0] > 0) if (MyValue[0] > MyValue[1]) PMomentumUp.Set(MyValue[0]); else PMomentumDown.Set(MyValue[0]);
						else  if (MyValue[0] > MyValue[1]) NMomentumUp.Set(MyValue[0]); else NMomentumDown.Set(MyValue[0]);

					}
					break;
				case SqueezeStyle.PBFSqueeze:
					Plots[0].Pen = new Pen(pbfsell, zerolinewidth);
					Plots[1].Pen = new Pen(pbfbuy,zerolinewidth);
					if (CurrentBar == 0) MyValue2.Set(Close[0]);
					else
					{
						MyValue2.Set(RGaussianFilter(Typical ,gaussperiod,gausspoles)[0]);
						if(MyValue2[0]>MyValue2[1]) SqueezeOn.Set(0); else Squeeze.Set(0);
						fv1 = RGaussianFilter(Typical, 8, 4)[0];
						sv1 = RGaussianFilter(Typical, 21, 4)[0];
						d1 = fv1 - sv1;
						fv2 = RGaussianFilter(Typical, 13, 4)[0];
						sv2 = RGaussianFilter(Typical, 34, 4)[0];
						d2 = fv2 - sv2;
						fv3 = RGaussianFilter(Typical, 21, 4)[0];
						sv3 = RGaussianFilter(Typical, 55, 4)[0];
						d3 = fv3 - sv3;
						fv4 = RGaussianFilter(Typical, 34, 4)[0];
						sv4 = RGaussianFilter(Typical, 89, 4)[0];
						d4 = fv4 - sv4;
						MyValue.Set((d1+d2+d3+d4)/4);
						Print(MyValue[0].ToString());
						if (MyValue[0] > 0) if (MyValue[0] > MyValue[1]) PMomentumUp.Set(MyValue[0]); else PMomentumDown.Set(MyValue[0]);
						else  if (MyValue[0] > MyValue[1]) NMomentumUp.Set(MyValue[0]); else NMomentumDown.Set(MyValue[0]);
						
					}
					break;
					
				default: 
					Plots[0].Pen = new Pen(pbfsell, zerolinewidth);
					Plots[1].Pen = new Pen(pbfbuy,zerolinewidth);

					if (CurrentBar == 0) MyValue2.Set(Close[0]);
					else
					{
						MyValue2.Set(RGaussianFilter(Typical ,gaussperiod,gausspoles)[0]);
						if(MyValue2[0]>MyValue2[1]) SqueezeOn.Set(0); else Squeeze.Set(0);
						fv1 = RGaussianFilter(Typical, 8, gausspolescounter)[0];
						sv1 = RGaussianFilter(Typical, 21, gausspolescounter)[0];
						d1 = fv1 - sv1;
						fv2 = RGaussianFilter(Typical, 13, gausspolescounter)[0];
						sv2 = RGaussianFilter(Typical, 34, gausspolescounter)[0];
						d2 = fv2 - sv2;
						fv3 = RGaussianFilter(Typical, 21, gausspolescounter)[0];
						sv3 = RGaussianFilter(Typical, 55, gausspolescounter)[0];
						d3 = fv3 - sv3;
						fv4 = RGaussianFilter(Typical, 34, gausspolescounter)[0];
						sv4 = RGaussianFilter(Typical, 89, gausspolescounter)[0];
						d4 = fv4 - sv4;
						MyValue.Set((d1+d2+d3+d4)/4);
						
						
						if (MyValue[0] > 0) if (MyValue[0] > MyValue[1]) PMomentumUp.Set(MyValue[0]); else PMomentumDown.Set(MyValue[0]);
						else  if (MyValue[0] > MyValue[1]) NMomentumUp.Set(MyValue[0]); else NMomentumDown.Set(MyValue[0]);
						
						if(CCI(cciperiod)[1]>50 && CCI(cciperiod)[0]<=50 && MyValue[0]>0 && MyValue[1]>0 && MyValue[2]>0) { NMomentumUp.Set(MyValue[0]); downct = 1; PMomentumUp.Reset(); PMomentumDown.Reset();}
						if(downct==1 && MyValue[0]>0 && CCI(cciperiod)[0]<100) { NMomentumUp.Set(MyValue[0]); PMomentumUp.Reset(); PMomentumDown.Reset(); }
						if(downct==1 && CCI(cciperiod)[1]<100 && CCI(cciperiod)[0]>100) { PMomentumUp.Set(MyValue[0]); downct=0;}
						if(downct==1 && MyValue[0]<0) downct=0;
						
						if(CCI(cciperiod)[1]<-50 && CCI(cciperiod)[0]>-50 && MyValue[0]<0 && MyValue[1]<0 && MyValue[2]<0) { PMomentumDown.Set(MyValue[0]); upct = 1; NMomentumUp.Reset(); NMomentumDown.Reset();}
						if(upct==1 && MyValue[0]>0) upct = 0;
						if(upct==1 && MyValue[0]<0 && CCI(cciperiod)[0]>-100) { PMomentumDown.Set(MyValue[0]); NMomentumUp.Reset(); NMomentumDown.Reset();}
						if(upct==1 && CCI(cciperiod)[1]>-100 && CCI(cciperiod)[0]<-100) { NMomentumDown.Set(MyValue[0]); upct=0; }
						
						
						
					}
					break;
			}
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Squeeze
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SqueezeOn
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries PMomentumUp
        {
            get { return Values[2]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries PMomentumDown
        {
            get { return Values[3]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries NMomentumUp
        {
            get { return Values[4]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries NMomentumDown
        {
            get { return Values[5]; }
        }

        [Description(" BB Squeeze ATR and StdDev Lenght")]
        [Category("BB Squeeze")]
		[Gui.Design.DisplayName ("ATR, StdDev Lenght")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }
		
        [Description("Zero Line Width")]
        [Category("Visual")]
		[Gui.Design.DisplayName ("Zero Line Width")]
        public int ZeroLineWidth
        {
            get { return zerolinewidth; }
            set { zerolinewidth = Math.Max(1, value); }
        }
		
        [Description("Histogram (Momentum) Width")]
        [Category("Visual")]
		[Gui.Design.DisplayName ("Histogram (Momentum) Width")]
        public int HistogramWidth
        {
            get { return histogramwidth; }
            set { histogramwidth = Math.Max(1, value); }
        }
		
        [Description("Bolinger Bands Std. Devs from Average")]
        [Category("BB Squeeze")]
		[Gui.Design.DisplayName ("Bolinger Bands Std. Devs from Average")]
        public double NBB
        {
            get { return nBB; }
            set { nBB = Math.Max(1, value); }
        }

        [Description("Keltner Channel ATRs from Average")]
        [Category("BB Squeeze")]
		[Gui.Design.DisplayName ("Keltner Channel ATRs from Average")]
        public double NK
        {
            get { return nK; }
            set { nK = Math.Max(1, value); }
        }

        [Description("Momentum Lenght")]
        [Category("BB Squeeze")]
		[Gui.Design.DisplayName ("Momentum Period Length")]
        public int MLength
        {
            get { return mLength; }
            set { mLength = Math.Max(1, value); }
        }
		
        [Description("CCI Period for CounterTrend Mode")]
        [Category("CounterTrend")]
		[Gui.Design.DisplayName ("CCI Period")]
        public int CCIPeriod
        {
            get { return cciperiod; }
            set { cciperiod = Math.Max(1, value); }
        }

        [Description("Gauss Filter Poles for Countertrend Mode")]
        [Category("CounterTrend")]
		[Gui.Design.DisplayName ("Gauss Filter Poles")]
        public int GaussPolesCounter
        {
            get { return gausspolescounter; }
            set { gausspolescounter = Math.Max(1, value); }
        }

       [Description("Gaussian Filter for PBF Period")]
        [Category("PBF Squeeze")]
		[Gui.Design.DisplayName ("Gaussian Filter Period")]
        public int GaussPeriod
        {
            get { return gaussperiod; }
            set { gaussperiod = Math.Max(1, value); }
        }

        [Description("Gaussian Filter for PBF Poles")]
        [Category("PBF Squeeze")]
		[Gui.Design.DisplayName ("Gaussian Filter Poles")]
        public int GaussPoles
        {
            get { return gausspoles; }
            set { gausspoles = Math.Max(1, value); }
        }

		[Description("Squeeze Style")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("Squeeze Style")]
        public SqueezeStyle SStyle
        {
            get { return sStyle; }
            set { sStyle = value; }
        }
		
        [Description("BBSqueeze  ZeroLine AlertColor")]
        [Category("BB Squeeze")]
		[Gui.Design.DisplayName ("BBSqueeze AlertColor")]
        public Color Alertcolor
        {
            get { return alertcolor; }
            set { alertcolor = value; }
        }

		[Browsable(false)]
		public string alertcolorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(alertcolor); }
			set { alertcolor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        [Description("BBSqueeze  ZeroLine NormalColor")]
        [Category("BB Squeeze")]
		[Gui.Design.DisplayName ("BBSqueeze NormalColor")]
        public Color Normalcolor
        {
            get { return normalcolor; }
            set { normalcolor = value; }
        }

		[Browsable(false)]
		public string normalcolorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(normalcolor); }
			set { normalcolor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        [Description("Positive Momentum Rising")]
        [Category("Visual")]
		[Gui.Design.DisplayName ("Positive Momentum Rising")]
        public Color Pmup
        {
            get { return pmup; }
            set { pmup = value; }
        }

		[Browsable(false)]
		public string pmupSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(pmup); }
			set { pmup = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        [Description("Positive Momentum Falling")]
        [Category("Visual")]
		[Gui.Design.DisplayName ("Positive Momentum Falling")]
        public Color Pmdown
        {
            get { return pmdown; }
            set { pmdown = value; }
        }

		[Browsable(false)]
		public string pmdownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(pmdown); }
			set { pmdown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        [Description("Negative Momentum Falling")]
        [Category("Visual")]
		[Gui.Design.DisplayName ("Negative Momentum Falling")]
        public Color Nmdown
        {
            get { return nmdown; }
            set { nmdown = value; }
        }

		[Browsable(false)]
		public string nmdownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(nmdown); }
			set { nmdown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Negative Momentum Rising")]
        [Category("Visual")]
		[Gui.Design.DisplayName ("Negative Momentum Rising")]
        public Color Nmup
        {
            get { return nmup; }
            set { nmup = value; }
        }

		[Browsable(false)]
		public string nmupSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(nmup); }
			set { nmup = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		

        [Description("PBF Buy Signal Color")]
        [Category("PBF Squeeze")]
		[Gui.Design.DisplayName ("PBF Buy Color (Zero Line)")]
        public Color PBFBuy
        {
            get { return pbfbuy; }
            set { pbfbuy = value; }
        }

		[Browsable(false)]
		public string pbfbuySerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(pbfbuy); }
			set { pbfbuy = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        [Description("PBF Sell Signal Color")]
        [Category("PBF Squeeze")]
		[Gui.Design.DisplayName ("PBF Sell Color (Zero Line)")]
        public Color PBFSell
        {
            get { return pbfsell; }
            set { pbfsell = value; }
        }

		[Browsable(false)]
		public string pbfsellSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(pbfsell); }
			set { pbfsell = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RSqueeze[] cacheRSqueeze = null;

        private static RSqueeze checkRSqueeze = new RSqueeze();

        /// <summary>
        /// PBF and BB Squeeze Replica
        /// </summary>
        /// <returns></returns>
        public RSqueeze RSqueeze(SqueezeStyle sStyle)
        {
            return RSqueeze(Input, sStyle);
        }

        /// <summary>
        /// PBF and BB Squeeze Replica
        /// </summary>
        /// <returns></returns>
        public RSqueeze RSqueeze(Data.IDataSeries input, SqueezeStyle sStyle)
        {
            checkRSqueeze.SStyle = sStyle;
            sStyle = checkRSqueeze.SStyle;

            if (cacheRSqueeze != null)
                for (int idx = 0; idx < cacheRSqueeze.Length; idx++)
                    if (cacheRSqueeze[idx].SStyle == sStyle && cacheRSqueeze[idx].EqualsInput(input))
                        return cacheRSqueeze[idx];

            RSqueeze indicator = new RSqueeze();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SStyle = sStyle;
            indicator.SetUp();

            RSqueeze[] tmp = new RSqueeze[cacheRSqueeze == null ? 1 : cacheRSqueeze.Length + 1];
            if (cacheRSqueeze != null)
                cacheRSqueeze.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRSqueeze = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// PBF and BB Squeeze Replica
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSqueeze RSqueeze(SqueezeStyle sStyle)
        {
            return _indicator.RSqueeze(Input, sStyle);
        }

        /// <summary>
        /// PBF and BB Squeeze Replica
        /// </summary>
        /// <returns></returns>
        public Indicator.RSqueeze RSqueeze(Data.IDataSeries input, SqueezeStyle sStyle)
        {
            return _indicator.RSqueeze(input, sStyle);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// PBF and BB Squeeze Replica
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSqueeze RSqueeze(SqueezeStyle sStyle)
        {
            return _indicator.RSqueeze(Input, sStyle);
        }

        /// <summary>
        /// PBF and BB Squeeze Replica
        /// </summary>
        /// <returns></returns>
        public Indicator.RSqueeze RSqueeze(Data.IDataSeries input, SqueezeStyle sStyle)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RSqueeze(input, sStyle);
        }

    }
}
#endregion
